
/* geo_dat.h                       # page  1   10 Feb 1995 11 23 */

/* Copyright (c) 1993 GD, Kort-og Matrikelstyrelsen, Denmark */
/* All rights reserved.                                       */

/* This is unpublished proprietary source code of GD, Kort-og */
/* Matrikelstyrelsen, Denmark.  This copyright claim does not */
/* indicate an intention of publishing this code.             */


#if !defined(H_GEO_DAT)
#define      H_GEO_DAT

#if !defined(H_MAX_OBS_TP)
#define      H_MAX_OBS_TP
#define    MAX_GG        30
#define    MAX_CB       100
#define    MAX_MN       300
#define    MAX_MN_OC    300
#endif       H_MAX_OBS_TP

#if !defined(H_GEO_LAB)
#include    "geo_lab.h"
#endif

#if !defined(BF_UL_DEF)
#define      BF_UL_DEF
struct     bf {
  unsigned int         xc: 1; /*   0 exists    */
  unsigned int         fc: 1; /*   1 fixed     */
  unsigned int         nc: 1; /*   2 new       */
  unsigned int         rc: 1; /*   3 requested */
  unsigned int         sc: 1; /*   4 buf stn   */
  unsigned int         oc: 1; /*   5 buf obj   */
  unsigned int         uc: 1; /*   6 unused    */
  unsigned int         ic: 1; /*   7 singular  */
  unsigned int         nn: 1; /*   8 natural p */
  unsigned int         cp: 1; /*   9 polygon   */
  unsigned int         pp: 1; /*  10 half bl p */
  unsigned int         cb: 1; /*  11 poly blind*/
  unsigned int         no: 1; /*  12 lost      */
  unsigned int         vt: 1; /*  13 vt3obs    */
  unsigned int         wk: 1; /*  14 work      */
  unsigned int         xh: 1; /*  15 exists    */
  unsigned int         fh: 1; /*   0 fixed     */
  unsigned int         nh: 1; /*   1 new       */
  unsigned int         rh: 1; /*   2 requested */
  unsigned int         sh: 1; /*   3 buf stn   */
  unsigned int         oh: 1; /*   4 buf obj   */
  unsigned int         uh: 1; /*   5 unused    */
  unsigned int         ih: 1; /*   6 singular  */
  unsigned int         se: 1; /*   7 secularmo */
  unsigned int         is: 1; /*   8 sing. se  */
  unsigned int         eh: 1; /*   9 ellh input*/
  unsigned int         hh: 1; /*  10 normalh in*/
  unsigned int         hp: 1; /*  11 polygon   */
  unsigned int         hb: 1; /*  12 poly blind*/
  unsigned int         gi: 1; /*  13 gravi inp */
  unsigned int         gr: 1; /*  14 req. gravi*/
  unsigned int         rg: 1; /*  15 req. geoid*/
};

union    bf_ul {
  struct bf        b;
  unsigned int     l;
};
#endif  BF_UL_DEF


/* geo_dat.h                       # page  2   10 Feb 1995 11 23 */


/* COORDINATES */
/* ----------- */
struct st_rec_str {
  double           Nc;         /*Northing          inp by ico*/
  double           Ec;         /*Easting           inp by ico*/
  double           Hc;         /*EllipsoidalHeight inp by ico*/
  double           Hn;         /*NormalHeight      inp by ico*/
  struct typ_dec   Ntpd;       /*typedec of coord*/
  struct typ_dec   Etpd;       /*typedec of coord*/
  struct typ_dec   Htpd;       /*typedec of height*/
  struct typ_dec   Hntpd;      /*typedec of geoidheight*/
  union bf_ul      sst;        /*station status*/
  double           NEcmt;      /*cmt of N and E*/
  double           Hcmt;       /*cmt of Hc*/
  int              Hyear;      /*measure-year(s) of Hn*/
  double           Hncmt;      /*cmt of Hn*/
};

/* HEADS */
/* ----- */
/* geometrisk geodsi + ured do : retninger, afstande, azimuth */
struct  gg_hd_str {
  int                o_stn;
  int                o_nme;
  short              o_kind;
  short              o_type;
  int                q_obj;
  int                year;
  int                sets;
  double             jsnr;
  struct typ_dec     otpd;
  double             md;
  struct typ_dec     md_tpd;
  double             mc;
  struct typ_dec     mc_tpd;
  double             spp;       /* special param */
  struct typ_dec     spp_tpd;   /* tpd for spp   */
};

/* koordinat anblock */
struct  stec_hd_str {
  int                o_nr;  /* date and o_nr is unique */
  int                observator; /* and instrument id. */
  short              o_kind;
  short              o_type;
  int                q_obj;
  int                sets;  /* antal PASPUNKTER til orientering */
  struct typ_dec     obs_tpd;
  short              orient; /* 0 = cl, 1 = acl, 2 = w */
  double             date;  /* obs date */
  double             mc;
  struct typ_dec     mc_tpd;
  double             mv;
  struct typ_dec     mv_tpd;
  double             x0; /* Orintation unknowns : zero */
  double             y0;
  double             h0;
};


/* geo_dat.h                       # page  3   10 Feb 1995 11 23 */

/* Photogrammetry : monocomparator mlinger */
struct  mono_hd_str {
  int                o_stn;
  int                o_nme;
  short              o_kind;
  short              o_type;
  int                q_obj;
  int                sets;  /* no.of remeasurements, in db == 1 */
  int                o_nr;
  int                cam_no;
  double             cam_f;  /* camera focus dist. */
  struct typ_dec     obs_tpd;
  short              orient;
  short              ram_ant;
  double             date;  /* obs date */
  double             mc;
  struct typ_dec     mc_tpd;
};

struct  niv_hd_str {
  int                o_stn;
  int                o_nme;
  short              o_kind;
  short              o_type;
  int                q_obj;
  int                year;
  int                sets;
  double             jsnr;
  struct typ_dec     otpd;
  double             i_height;
  struct typ_dec     ih_tpd;
  double             md;
  struct typ_dec     md_tpd;
  double             mc;
  struct typ_dec     mc_tpd;
  double             spp;
  struct typ_dec     spp_tpd;
};


/* geo_dat.h                       # page  4   10 Feb 1995 11 23 */


/* LINES */
/* ----- */
struct  gg_ln_str {
  int                obj;
  int                nme;
  double             obs;
};

/* vektorer fra gps */
struct  vt3_ln_str {
  int                obj;
  int                nme;
  double             obsN;
  double             obsE;
  double             obsH;
  double             QXX;
  double             QXY;
  double             QYY;
  double             QXZ;
  double             QYZ;
  double             QZZ;
  double             DET;
};

struct  stec_ln_str {
  int                obj;
  int                nme;
  double             obs_y;
  double             obs_x;
};

struct  st3c_ln_str {
  int                obj;
  int                nme;
  double             obsX;
  double             obsY;
  double             obsZ;
};

union   stec_mn_str {
  struct  stec_ln_str u_st;
  struct  st3c_ln_str u_3c;
};

struct  mono_ln_str {
  int                q_obj;
  int                nme;
  double             date;
  char               opr[4];
};

union   mono_mn_str {
  struct  mono_ln_str u_mn;
  struct  stec_ln_str u_st;
};

struct  niv_ln_str {
  int               obj;
  int               nme;
  double            obs;
  double            sln;
  struct typ_dec    ltpd;
  double            obs_dt;
  short             set_u;
  float             eta;
};


/* geo_dat.h                       # page  5   10 Feb 1995 11 23 */


/* STATION - head structures */
/* ------------------------- */
struct  gg_hd_st_str {
  struct st_str        st_nm;
  struct gg_hd_str     gg_hd;
};

struct  niv_hd_st_str {
  struct st_str      st_nm;
  struct niv_hd_str  ni_hd;
};

struct  stec_hd_st_str {
  struct st_str        st_nm;
  struct stec_hd_str   st_hd;
};

struct  mono_hd_st_str {
  struct st_str        st_nm;
  struct mono_hd_str   mn_hd;
};

/* STATION - line structures */
/* ------------------------- */
struct  gg_ln_st_str {
  struct st_str        st_nm;
  struct gg_ln_str     gg_ln;
};

struct  niv_ln_st_str {
  struct st_str      st_nm;
  struct niv_ln_str  ni_ln;
};

struct  vt3_ln_st_str {
  struct st_str        st_nm;
  struct vt3_ln_str    vt_ln;
};

struct  stec_ln_st_str {
  struct st_str        st_nm;
  struct stec_ln_str   st_ln;
};

struct  st3c_ln_st_str {
  struct st_str        st_nm;
  struct st3c_ln_str   st_3c;
};

struct  stec_mn_st_str {
  struct st_str        st_nm;
  union  stec_mn_str   st_mn;
};

struct  mono_mn_st_str {
  struct st_str        st_nm;
  union  mono_mn_str   mn_mn;
};


/* geo_dat.h                       # page  6   10 Feb 1995 11 23 */


/* OBSERVATION - unions */
/* -------------------- */
union  obs_hd_str {
  struct  gg_hd_st_str   u_gg_hd;
  struct  mono_hd_st_str u_mn_hd;
  struct  niv_hd_st_str  u_ni_hd;
  struct  stec_hd_st_str u_st_hd;
};

union  obs_ln_str {
  struct  gg_ln_st_str   u_gg_ln;
  struct  mono_mn_st_str u_mn_ln;
  struct  niv_ln_st_str  u_ni_ln;
  struct  stec_ln_st_str u_st_ln;
  struct  st3c_ln_st_str u_3c_ln;
  struct  stec_mn_st_str u_3t_ln;
  struct  vt3_ln_st_str  u_vt_ln;
};

/* monocomparator picture OC */
struct  mono_OC_str {
  struct  st_str     ph_nr;
  double             o_date;    /* obs date */
  int                status;
  char               c_lab[16]; /* coord_lab */
  double             c_N;       /* camera pos N */
  double             c_E;       /* camera pos E */
  double             c_H;       /* camera pos H */
  double             rot_x;     /* camera rot cam-x_axis */
  double             rot_y;     /* camera rot cam-y_axis */
  double             rot_z;     /* camera rot cam-z_axis */
  double             c_cmt;     /* cmt of OC */
  struct typ_dec     pos_tpd;
  struct typ_dec     rot_tpd;
  int                p_oc;      /* add til pointer oc_base */
};

struct  lab_mn_OC_str {
struct  coord_lab    lab_mono;
struct  mono_OC_str  mono_OC;
};

struct srch_str {
int                  nmb;
int                  cix;
union stn_pck        ust;
};

/* OBSERVATION - kind_type */
/* ----------------------- */
#define   GG_DIR       101
#define   GG_LAP       102
#define   GG_DST       103
#define   GG_BMD       104
#define   GG_MIX       110
#define   GG_VT3       112

#define   RG_DIR       201
#define   RG_LAP       202
#define   RG_DST       203
#define   RG_BMD       204
#define   RG_MIX       210

#define   NI_PRS       305
#define   NI_NIV       306
#define   NI_ZDS       307
#define   NI_MTZ       308
#define   NI_POT       309

#define   ST_CL_       401
#define   ST_ACL       402
#define   ST_W__       403
#define   ST_HN_       404
#define   ST_HE_       405
#define   ST_HS_       406
#define   ST_CS_       407
#define   ST_DATUM     409

#define   MN_CL_       501
#define   MN_ACL       502
#define   MN_RHS       503

#define   MEAN_CL_    5501
#define   MEAN_ACL    5502
#define   MEAN_RHS    5503



/* geo_dat.h                        # page 7   25 Jan 1996 09 02 */


/* SPECIAL: to be used in database function and programs */

/* define max number of obslines in union */
#define   MAD_OBSLINE    1100 

/* define max number of obsheads in union */
#define   MAD_OBSHEAD     600 

/* define max number of obs_id's saved in dmut_xxx func-  */
/* tions, used to avoid returning the same set_up more    */
/* than once, one for every function.                     */
#define   MAD_VT3_ID      200 
#define   MAD_GG2_ID     1200 
#define   MAD_NIV_ID      200 

/* define max number of obs_id's saved in dget_obs for cb */
/* used to stop returning the same block more than once   */
#define   MAD_CBHEADS     200 

/* define max number of obs_id's for a station returned  */
/* by dstn_2_cb, used to setup select in dget_obs for cb */
#define   MAD_CBOBSID      10 

/* define max number of jnsd_numbers in parameter call */
#define   MAD_JNSDCAL      50

/* define max number of labels in parameter call  */
#define   MAD_LABELS       10

/* define max number of DStstrCrd in function call  */
/* (history select)                                 */
#define   MAD_CRDREC       20

/* define number of DStstrCrd and DStstrIdt structures */
/* for malloc call (allocation in blocks of this size) */
#define   CRDIDT_BLK      512
/* define number of DStstrCrd and DStstrIDT structures */
/* returned in one array fetch                         */
#define   CRDARRAY_FETCH   64
#define   IDTARRAY_FETCH  128

/* define number of DStstrBsk structures               */
/* for malloc call (allocation in blocks of this size) */
#define   BSKBLK_SIZE     256

/* define max size in bytes for text-description  */
#define   MAD_BSKSIZE     768


/* use the above values in declarations, many of    */
/* these values are used for index control in data- */
/* base functions.                                  */

/* define type of connection                      */
/* used to decide where to put message file, and  */
/* the name of this file.                         */
#define   NORMAL_CONNECT    1
#define   UPDATE_CONNECT    2
#define   USR_CONNECT       3
#define   WEB_CONNECT      10
#define   WEB_ENV          10



/* geo_dat.h                        # page 8   25 Jan 1996 09 02 */


/* COORDINATES return from database */
/* -------------------------------- */

struct d_crd_rec {
  double          N_c;         /* Northing */
  double          E_c;         /* Easting */
  double          H_c;         /* Height */
  struct typ_dec  c_tpd;       /* typedec of crd (NE) */
  struct typ_dec  h_tpd;       /* typedec of hgt (H)  */
  double          c_cw;        /* comment word of crd */
  double          h_cw;        /* comment word of hgt */
  double          c_mw;        /* md comment word of crd */
  double          h_mw;        /* md comment word of hgt */
  int             desc_bits;   /* crd or hgt descriptors */
};

struct db_rec_str {
  double           N_c;         /* Northing                     */
  double           E_c;         /* Easting                      */
  double           H_c;         /* E_height or N_height         */
  struct typ_dec   N_tpd;       /* typedec of N_c               */
  struct typ_dec   E_tpd;       /* typedec of E_c               */
  struct typ_dec   H_tpd;       /* typedec of H_c               */
  double           c_cw;        /* comment word of crd          */
  double           h_cw;        /* comment word of H_c          */
  double           c_mw;        /* md comment word of crd       */
  double           h_mw;        /* md comment word of H_c       */
  int              H_year;      /* measure-year(s) of H_c       */
  int              desc_bits;   /* crd or hgt descriptors       */
};
 
struct DStstrCrd    {
  union stn_pck     st_nm;
  struct d_crd_rec  cr_ln;
  struct db_rec_str db_ln;
};

struct DStstrIdt    {
  int               idt_found;
  union stn_pck     idt_st[6];
};

struct DStstrGrp    {
  int               grp_found;
  union stn_pck     grp_st;
  int               carrier;
};

struct DStstrBsk    {
  struct st_str     st_nm;   
  char              st_bsk[MAD_BSKSIZE]; 
  int               refnr;     
  short             versnr;   
  char              format[12];  
};

/* not used for the moment */
struct dStstrBsk    {
  union stn_pck     st_nm;
  int               desc_bits;   /* only stn descriptors */
  int               next_bsk;    /* adr. or offset       */
};



/* geo_dat.h                        # page 9   25 Jan 1996 09 02 */


/* structures holding information in communication */
/* between programs and database funtions.         */
/* under construction frs mar. 1997                */

struct   call_message {
  /* Type of connection                              */
  /* ConnectWDB acts upon this value.                */
  /* Set connect_type in calling program to:         */
  /* UPDATE_CONNECT if you want to insert, delete or */
  /*   update in any tables.                         */
  /* NORMAL_CONNECT or                               */
  /* WEB_CONNECT if you want to select information.  */

  /* NORMAL_CONNECT will automaticly be set if       */
  /* !(USR_CONNECT or WEB_CONNECT or UPDATE_CONNECT) */
  int                 connect_type; 
  int                 prog_environ; 

  /* Message_file_name and file pointer              */
  /* if NORMAL_CONNECT or UPDATE_CONNECT, ConnectWDB */
  /*   will generate the name and open a file.       */
  /* if WEB_CONNECT, no message_file but stdout.     */
  char                message_file[56]; 
  FILE               *f_message; 

  /* After the 'connect to data_base message', ftell */
  /* is called. If (f_message -f_start_m) > 0L then  */
  /* the disconnect function will make a note, that  */
  /* a message exist in the message_file             */
  long                f_start_m;  

  /* Print or not the SQL-statement in message_file  */
  int                 SQL_print;

  /* Next some values used to send parameters from   */
  /* the program to the functions.                   */

  /* Method of measurement for HGT (in select call)  */
  /* must be set in calling program or function      */
  /* 1 = ter, 3 = GPS, 4 == picture..                */
  /* init in ConnectWDB to 1 (terr.)                 */
  int                 HGT_method; 

  /* Station in 'bane_ref' table (in select call)    */
  int                 bane_req;

  /* Stationtype (in select call)                    */
  /* must be set in calling program or function      */
  /* 0 = information not used, 1 = G.I. station      */
  /* 2 = M.V. station          4 = Eks station       */
  /* Or sum of these                                 */
  int                 stat_status; 

  /* Stationfilter (in select call)                  */
  /* must be set in calling program or function      */
  /* descriptor neg. mask.                           */
  int                 stat_filter; 

  /* Mask for station number type (in select call)   */
  /* See geo_lab.h for types.                        */
  /* Positive and negative mask (as bitmask)         */
  short               st_pos_mask;
  short               st_neg_mask;

  /* Protectet station (in select call)              */
  /* stations with 'r' will not be selectet if set   */
  /* init in ConnectWDB to 0 (unprotectet)           */
  /* set in calling program if otherwise wanted      */
  int                 r_protect; 

  /* Select since this date_time (in select call)    */
  /* this value can be used when selecting CRD or    */
  /* HGT since a certain date, init in ConnectWDB    */
  /* to 0.0, which means that the value is not used, */
  /* set in calling program if otherwise wanted      */
  double              since_date;

  /* Compute mean of mono-observations when fetching */
  /* these from the database.                        */
  int                 monomean;

  /* Select main station number, no matter what the  */
  /* calling number is.                              */
  int                 mainstn;

  /* Format description text for station to descrip- */
  /* tion sheet.                                     */
  /* init in ConnectWDB 0, which means that the      */
  /* value is not used, set in calling program if    */
  /* otherwise wanted                                */
  int                 desc_sheet;

  /* jointable is for special costumer things in     */
  /* Valdemar.                                       */
  char                jointable[8]; 

  int                 intern; 
  /* not used */
  int                 call_value_2; 
  /* not used */
  int                 call_value_3; 

  /* text that will be printed when rollback is called */
  char               *rb_txt; 
};



/* geo_dat.h                        # page 9   25 Jan 1996 09 02 */


struct   return_message {
  /* If we are connected to the database,   */
  /* ConnectWDB will set this variable to 1 */
  /* and DisConnectWDB will reset to 0      */
  int                 connect_state;

  /* Application state is increased every   */
  /* time ConnectWDB is called, the database */
  /* functions uses this value to see if    */
  /* they need to setup SQL-statements.     */
  int                 appl_state;

  /* Database name from ConnectWDB          */
  char                database_name[48];

  /* Last function-name, used in error-func */
  char                last_function[56]; 

  /* If transaction data_base, set in ConnectWDB     */
  int                 transaction;  
  int                 trans_state;

  /* date_time in insert functions, set first time called */
  double              date_time;   

  /* insert count(tables), set in each call to function */
  int                 nb_insert;

  /* update count(tables), set in each call to function */
  int                 nb_update; 

  /* delete count(tables), set in each call to function */
  int                 nb_delete; 

  /* insert/update location crd or hgt in junk tabel    */
  int                 jt_insert; 
  int                 jt_update; 

  /* Are we working with an Intel processor ? */
  int                 intel_pro;

  /* Are we in Windows environment ? */
  int                 windows_32;

  /* for some functions, this value is used to indicate */
  /* the relation between selected and returned values  */
  double              hit_rate;
  /* not used */
  int                 retn_value_3; 
};

struct  fnc_message {
  struct   call_message     ca_ms;
  struct   return_message   re_ms;
};

#if !defined(STR_LNG64)
#define      STR_LNG64      64
#endif
#if !defined(LNG_RAPMESS)
#define      LNG_RAPMESS  4096
#endif

/* Structure holding environment of different kind. */
struct webConfig {
  /* WEB SERVER environment */
  char            KMSHomePage[STR_LNG64];
  char            ImageDir[STR_LNG64];
  char            JavaScriptDir[STR_LNG64];
  char            StyleSheetDir[STR_LNG64];
  char            HtdocsDir[STR_LNG64];
  char            DownloadDir[STR_LNG64];
  char            DownloadPath[STR_LNG64];
  char            WorkDir[STR_LNG64];
  char            WorkPath[STR_LNG64];
  char            CgiDir[STR_LNG64];
  char            CgiPath[STR_LNG64];
  char            TempDir[STR_LNG64];
  char            LogsPath[STR_LNG64];
  /* GEOIDE DATA (?) */
  char            GeoidDataPath[STR_LNG64];
  /* DCW DATA */
  char            DCWDataPath[STR_LNG64];
  /* Read the ZIP PROGRAM */
  char            ZipProg[STR_LNG64];
  char            ZipOptions[STR_LNG64];
  /* Read the GZIP PROGRAM */
  char            GzipProg[STR_LNG64];
  char            GzipOptions[STR_LNG64];
  char            UnZipPath[STR_LNG64];
};


#endif


