

/* lin_stec.h                     # page 1   09 Jan 1995 10 58 */


/* Copyright (c) 1993 GD, Kort-og Matrikelstyrelsen, Denmark   */
/* All rights reserved.                                        */

/* This is unpublished proprietary source code of G/D, Kort-og */
/* Matrikelstyrelsen, Denmark.  This copyright claim does not  */
/* indicate an intention of publishing this code.              */


extern int               lin_stec(
/*______________________________*/
FILE                    *fw,
struct gst_str          *gst,
double                 **p_oc,
struct ci_str           *ci_base,
struct nl_ca_str        *nl_cat,
double                  *NL,
int                      eq_wanted
);

/* coordinat_mono_obs kun i plant koordinatsystem */

/*     MODEL  :

    CV = C0 + MR  * (MC - MC0)

    CV  is Coordinate Vector (N, E)
    C0  is weighted mean of coordinates CV : (N0, E0)
    MR  is inverse of scale and rotation matrix in rhs_stec
    MC  is Model Coordinate  (y, x)
    MC0 is weighted mean of Model Coordinate MC : (y0, x0)

    CV  is pointed by p_ci
    MC  is pointed by stec_ln->obs_y, stec_ln->obs_x
    MC0 is pointed by stec_hd->y0, stec_hd->x0
    N0  is pointed by *p_oc
    E0  is pointed by *p_oc+1
    Rotation       by *p_oc+2
    Scale          by *p_oc+3

    The equation is solved for unknowns OC, and the obs-eq's
    are returned in C_base in order : n, e
    The righthandside is returned in W_base in order : W, P, Rf
    where P are the actual weights
    and Rf the possible ReductionFactor

  */



/* lin_stec.h                     # page 2   09 Jan 1995 10 58 */


  /*                       { cos a  -sin a }    { alfa  -beta }  */
  /* MR  =1/ *(*p_oc +3) * {               }  = {             }  */
  /*                       { sin a   cos a }    { beta   alfa }  */
  /*  a  = *(*p_oc+2)                                            */

  /*  the model righthandside is :

    RHS = (CV - C0) - MR  * (MC - MC0)

    the model lefthandside is : A, giving the normal equation
    N  =  AT  *  Q  *  A, where AT is A transposed and Q is the
    inverse of the weightmatrix.

    AT = (     1     ,     0     )         ( d N0   )
         (     0     ,     1     )         ( d E0   )
         (   y - y0  ,   x - x0  )         ( d alfa )
         ( -(x - x0) ,   y - y0  )         ( d beta )
         (    -1     ,     0     )         ( d N(i) )
         (     0     ,    -1     )         ( d E(i) )

    d *(p +2) = (-beta * d alfa + alfa * d beta )* *(p +3)* *(p +3)
    d *(p +3) = (-alfa * d alfa - beta * d beta )
    _            * *(p +3)* *(p +3)* *(p +3)
    
    The 4 first Orientation Constants may be locally elliminated
    on the known station coordinates.
    Thereafter is to be returned to the obs_neq function :
       the coefficients to d N(i) and d E(i), and
       the OC coefficients (AT4 * A2L / REDUCED( AT4 * A4 ))(i).
    AT4 is the first 4 rows in AT, and A2L the 2 last cols in A.

    AT4 * A, AT4 * W = (OC_a0    0.0    OC_a1  -OC_a2,  OC_a4)
                       (        OC_a0   OC_a2   OC_a1,  OC_a5)
                       (                OC_a3    0.0 ,  OC_a6)
                       (                        OC_a3,  OC_a7)

    For obj no correlation with OC gives diagonal contribution only.

    NOTE : ALL KNOWN COORDINATES ARE "FIXED" IN LINEAR MODE.
    NOTE : HENCE NO CORRELATION WITH OC IS TAKEN IN ACCOUNT.

  */

