

/* o_redun                          # page 1    9 Nov 1995 17 19 */


/* Copyright (c) 1993, GD, Kort-og Matrikelstyrelsen, Denmark  */
/* All rights reserved.                                        */

/* This is unpublished proprietary source code of G/S, Kort-og */
/* Matrikelstyrelsen, Denmark.  This copyright claim does not  */
/* indicate an intention of publishing this code.              */


extern double       o_redun(
/*________________________*/
FILE               *fw,
double             *C_base,
struct gst_str     *gst,
int                 stn,
int                 q_obj,
int                 eq_p_obj,
int                 cf_p_stn,
int                 oc_ps,
double             *NL_inv,
int                *NL_col,
int                *NL_ix0,
struct rdu_str     *rdu_base
);

/* o_redun calculates local redundancy to a round of observations */
/*         to q_obj object and stn (=0/1) station,                */
/*         with oc_ps Orientation unknowns (prior eliminated)     */

/* stn == 0 :  no station in obs  (q_obj1 = q_obj)    */
/* stn != 0 : one station in obs  (q_obj1 = q_obj+1)  */

/* eq_p_obj == equations pr. object     */
/* cf_p_stn == coeffs    pr. station    */
/* oc_ps    == no of Orientations       */
/* NL_inv   == inverse normal equations */
/* NL_col   == obs to col index         */
/* NL_ix0   == index of NL[col, 0]      */

/* K. Engsager.  Nov. 1995 */


/* o_redun                          # page 2    9 Nov 1995 17 19 */


/* ORGANIZATION of NL_col and NL_ix0 */
/* NL_col [q_obj1 * cf_p_stn + 1] , the last is RHS_COL      */
/* NL_ix0 [NL_ix0, order, i_order] :                         */
/* NL_ix0 [q_obj1 * cf_p_stn + 1] , the last is RHS_COL      */
/*        (0:q_obj-1) is col_ix0 of object                   */
/* if (stn) (q_obj)   is col_ix0 of station                  */
/*          (q_obj1)  is col_ix0 of RHS_COL                  */
/* order  [q_obj1 * cf_p_stn + 1] , new_ix = order[old_ix]   */
/* i_order[q_obj1 * cf_p_stn + 1] , old_ix = i_order[new_ix] */

/* ORGANIZATION of R and C */
/* C[C_co, C_cs, C_oc, R_oc, Owrk] :                             */
/*   if (stn   == 0) is C_cs absent !                            */
/*   if (oc_ps == 0) is C_oc absent !                            */
/* C_co, C_cs[cf_p_stn * eq_p_obj * q_obj]                       */
/*   cf_p_stn == 1: C_co_h, C_cs_h[eq_p_obj * q_obj]             */
/*   cf_p_stn == 2: C_co_n, C_co_e, C_cs_n, C_cs_e[eq_p_obj * q_obj] */
/*   cf_p_stn == 3: C_co_n, C_co_e, C_co_h,                      */
/*                  C_cs_n, C_cs_e, C_cs_h[eq_p_obj * q_obj]     */
/* C_oc[cf_p_stn * oc_ps * (q_obj + stn)]                        */
/*   if (stn   == 0) is C_oc absent !                            */
/*   cf_p_stn == 1: C_oc_h[oc_ps * q_obj], C_os_h[oc_ps]         */
/*   cf_p_stn == 2: C_oc_n[oc_ps * q_obj], C_os_n[oc_ps],        */
/*                  C_oc_e[oc_ps * q_obj], C_os_e[oc_ps]         */
/*   cf_p_stn == 3: C_oc_n[oc_ps * q_obj], C_os_n[oc_ps],        */
/*                  C_oc_e[oc_ps * q_obj], C_os_e[oc_ps]         */
/*                  C_oc_h[oc_ps * q_obj], C_os_h[oc_ps]         */
/* R_oc[oc_ps * eq_p_obj * q_obj] : oc_coef_matrix (unreduced)   */
/*  ([oc_ps])    elements in each oc_line                        */
/* Owrk[q_obj1 * cf_p_stn] , workarea for OC_elim                */

/* C_oc and R_oc is reduced with the normal matrix of the OC's : */
/*                                                               */

/*  if (!oc_ps) */
/* LR = Local Redundancy                                       */
/* A = [C_co C_cs], t index is row t (or col), T is transposed */
/* LR = 1.0 - At * N_inv * AtT                                 */


/*  if (oc_ps) */
/* LR = Local Redundancy                                        */
/* OBS_eq :  [P A] * [OC X] = W                                 */
/* A  = [C_co C_cs], t index is row t, T is transposed          */
/* P  = R_oc : Reduced Oc coefficients                          */
/* B  = C_oc : Reduced [AT * Oc] coefficients                   */
/* Dt = At - Pt * B                                             */
/* LR = 1.0 - Pt * PtT - Dt * N_inv * DtT                       */

