

/* set_grs.h version 1.0            # page 1   26 Jul 1994 18 25 */


/* Copyright (c) 1989 U/32, Kort-og Matrikelstyrelsen, Denmark */
/* All rights reserved.                                        */

/* This is unpublished proprietary source code of G/S, Kort-og */
/* Matrikelstyrelsen, Denmark.  This copyright claim does not  */
/* indicate an intention of publishing this code.              */

#ifndef      _SET_GRS_H
#define      _SET_GRS_H

extern int          set_grs(
/*________________________*/
int          ell_nmb,
char         *ellname,
double       *e,
...
);

/* 
Function:

    set_grs initializes the parameters of the reference ellipsoid
as a function of the string ellname. The names and KMS-numbers
of the ellipsoids are shown in the macro definitions above.

    The original definitions are used for each ellipsoid, but for
the older ellipsoids (e.g. Hayford) the equator gravity value and
the standard rotation velocity is used to enable a full definition. 

Call and parameters:

set_grs   (integer, return value).
The actual number of the initialized ellipsoid. The numbers can 
be found by running grs_doc. NBNBNB any call will initialize the 
ellipsoid, even the ?, #, and ! without name and number, which 
gives the number -1 and puts e[0] = 0.0. This is also the result 
from an unsuccesfull call, reported as "mode 0 is illegal" on stderr.

ellname  (string, call value). 
The name of the ellipsoid.  The number of the ellipsoid in
character representation can also be used.

e        (double e[9],  (call and) return value).
e[0] = a;    semi-major axis of the reference ellipsoid
e[1] = f;    flattening (a-b)/a, a/(a-b) can be used in a contingent
             call, but (a-b)/a is returned also in this case.
e[2] = J2;   dynamical form factor
e[3] = GM;   gravitational constant * mass
e[4] = omg;  angular velocity [radians/sec]
e[5] = U0;   normal potential at the ellipsoid
e[6] = geq;  normal gravity at equator
e[7] = ksf;  k from Somigliana's formula
e[8] = Q;    The meridian quadrant.

Prog: Maja Boserup, Knud Poder May-Oct 1990
Updated: Knud Poder NOV 1992

*/


/* set_grs.h version 1.0            # page 2   26 Jul 1994 18 25 */


/* 
User-defined ellipsoid:

    The user may define an ellipsoid during the execution of a pro-
gramme. This definition is valid until the termination of the pro-
gramme, or until the same programme produces a new definition.

The definition is carried out by calling set_grs with the ellip-
soid name string

                   #<mode><NAME> 

where mode and the necessary parameters in the call are:
     mode    e[0]   e[1]   e[2]   e[3]   e[4]   e[5]   e[6]
      1      a             J2     GM    omega     
      2      a      f                   omega          geq
      3      a      f             GM    omega     
      4    mean dg  f             GM    omeg

The defining constants according to the wanted mode are assigned
to the elements of e according to the table. For GM, geq and 
omega any negative number will signal that the standard values are
used, viz. GM from GRS80 or geq from Hayford 1924 as required for
the mode, and the standard value of omega for the earth, unless
the user assigns a new value. All values of e are available after 
the call, except for the special info-calls, where e[0] = 0.0 and 
set_grs returns -1. <NAME> may be empty, resulting in the ellipsoid 
name  "Special".

Documentation may be obtained by calls with ellname

     !<NAME or NUMBER>   =>  Ellipsoid name, and KMS-number   
     ?<NAME or NUMBER>   =>  Ell. name, nmb, and constants
     *<NAME or NUMBER>   =>  Ell. name, nmb, and const. for mainten.

Calling with ?, #, or ! only produces information for all 
ellipsoids. The programme grs_doc may be used for online docu-
mentation. Use <CR> after prog prompt "grs_doc" for instruction.

*/

#endif

