

/* set_mc.h                         # page 1   21 Nov 1996 13 25 */


/* Copyright (c) 1996 GD, Kort-og Matrikelstyrelsen, Denmark   */
/* All rights reserved.                                        */

/* This is unpublished proprietary source code of GD, Kort-og  */
/* Matrikelstyrelsen, Denmark.  This copyright claim does not  */
/* indicate an intention of publishing this code.              */


#if !defined       (H_SET_MC)
#define             H_SET_MC

extern void  set_mc(
double             *lampda,
double             *cs,
int                 nr,
double             *ar
);


  /*   initialisering af meteorologikonstanter i arrayet ar.     */
  /*   ++++++++++++++++++++++++++++++++++++++++++++++++++++      */
  /*           Parametre:                                        */
  /*             lampda (call value, real). Breblgelngde.     */
  /*             cs     (call value, real). Standard hastighed.  */
  /*             nr     (call value, int). Instrumentnummer.    */
  /*             ar     (return value, array). Arrayet(-6:26)    */
  /*                    indeholder meteorologi-konstanter.       */

  /* BEMRK : ar(-6:26) ndret til ar[33] (C deklaration)        */

  /* proceduren set_mc_tx oversat til set_mc.c                   */
  /* af FRS 21. november 1996                                    */

	

/* set_mc.c                         # page 2   21 Nov 1996 13 25 */


  /*     Memo  vedr.  afstandsreduktioner.                           */

  /*  Der mles principielt en fase forskydning mellem udsendt og    */
  /*  modtaget signal :                                              */
  /*  2 * ( 2 * pi * F * t)                                          */

  /*  Den diffentielle faseforskydning er da                         */
  /*  dA = d(4*pi*F*t)  = 4 * p * ds / lambda                        */
  /*  = 4 * pi * ds * F / ca                                         */
  /*  = 4 * pi * (ds / cs) * (na / ns) * F                           */
  /*  = 4 * pi ds * (F / cs) * (ns + na - ns)/ns                     */
  /*  = 2 * pi * ds * (F / cs) * (1 + (na - ns)/ns)                  */

  /*  hvor     lambda er den aktuelle modulans blgelngde           */
  /*  ca     er den aktuelle udbredelseshastighed                    */
  /*  cs     er den valgte   udbredelseshastighed                    */
  /*  na     er det aktuelle brydningsindeks                         */
  /*  ns     er det til cs svarende brydningsindeks                  */

  /*  Heraf fs                                                      */

  /*  ds = dA / (4 * pi) * (cs / F) * (1 - (na - ns)/ns)             */
  /*  og                                                             */
  /*  S = cs / (4 * pi * F) integral (1 - (na - ns)/ns) * dA         */
  /*  = cs / (4 * pi * F) * (A - integral(na-ns)/ns * dA)            */
  /*  = cs / (4 * pi * F) * (A - M)                                  */

  /*  hvor      M     er den meteorologiske korrektion i afstandsml */

  /*  Endvidere udvikles                                             */

  /*  S + M = cs / (4 * pi * (Fs + F - Fs)) * A                      */
  /*  = cs / (4 * pi * Fs(1 + (F - Fs)/Fs)) * A                      */
  /*  = cs / (4 * pi * Fs) * A * (1 - (F - Fs)/Fs)                   */

  /*  hvor     Fs     er den valgte standardfrekvens                 */

  /*  VIGTIG BEMRKNING : Fs kan vlges uafhngig af cs !            */

  /*  Vlges   Fs     som instrument standardfrekvens fs            */

  /*  S + M = aflsning * (1 - (F - Fs)/Fs)                          */

  /*  Instrumentets standard brydningsindeks kan beregnes ud fra     */
  /*  instrumentets standardhastighed cs                             */

  /*  ns = c0 / cs = 1 + (c0 - cs) / cs                              */
  /*  hvor c0 er lyshastigheden i vakuum.                            */

	
#endif

