
/* typ_dec.h  version 1.1           # page 1   22 Feb 1994 13 13 */


/* Copyright (c) 1992 GS, U/32, Kort-og Matrikelstyrelsen, Denmark */
/* All rights reserved.                                        */

/* This is unpublished proprietary source code of G/S, Kort-og */
/* Matrikelstyrelsen, Denmark.  This copyright claim does not  */
/* indicate an intention of publishing this code.              */



#if !defined(H_TYP_DEC)
#define      H_TYP_DEC

/*   Defined in geo_lab.h
struct typ_dec {
  unsigned char   gf;
  unsigned char   tf;
  unsigned char   ff;
  unsigned char   df;
};
*/

/*
typ_dec.h contains the source code of declarations used by 
the functions set_tpd and get_tpd.

The information is a structure array containing

1.  the name of the unit
2.  the group of the unit
3.  the type of the unit
4.  the conversion factor of the unit

The name is a letter string of 1-4 characters identifying
the unit.

The groups are:

1. sexagesimal angular values
type 1: degrees, minutes and (arc)seconds               (sx)
type 2: hours, minutes and (time)seconds                (tm)
type 3-6: as type 1, but N,S,E,W are available      (N) (S)
          for convenience, S and W means negative   (E) (W)
          values.
type 7: not used at present
type 8: degrees and minutes                             (nt)
type 9: mili(degrees, minutes and (arc)seconds)         (mas)

2. decimal angular values 
type 1: nonagesimal angles (old-degrees with decimals)  (dg)
type 2: arcseconds                                      (asx)
type 3: radians                                         (rad)
type 4: microradians  (1.0e-6 rad)                      (yrad)
type 5: centesimal degrees, minutes and seconds         (cc)
type 6: gon                                             (gr)
type 7: gon                                             (gon)
type 8: mgon                                            (mgon)
type 9: ts (="thousands") 4000 ts is 2 pi               (ts)
type 10: picoradians  (1.0e-9 rad)                      (prad)

3. not used
*/


/* typ_dec.h  version 1.1           # page 2   22 Feb 1994 13 13 */


/*

4. lengths units
type 1: metres                                       (m)
type 2: microns                                      (my)
type 3: millimetres                                  (mm)
type 4: centimetres                                  (cm)
type 5: decimetres                                   (dm)
type 6: kilometres                                   (km)
type 7: nautical miles 1 nm = 1852.0 m               (nm)

5. astronomical units
type 1: astronomical unit in meters                  (aum)
              149 597 870 691 m
type 2: astronomical unit in seconds                 (aus)
                          499.004 783 806 1 sec 
type 3: lysr                                        (aly)
        9 460 730 472 580 800 m

6. - 7. not used

8. potential units
type 1: gpu, geopotential units (appr. 0.98 m)       (gpu)
type 2: milli-gpu                                    (mgpu)
type 3: micro-gpu                                    (ygpu)

9. not used

10. gravity units
type 1: gal    (cm/sec2)                             (gal)
type 2: milligal                                     (mgal)
type 3: microgal                                     (ygal)
type 4: kilogal                                      (kgal)

11. not used

12. mean error units
type 1: ppm   (parts pr million)                     (ppm)
type 2: ppb   (parts pr billion)                     (ppb)
type 3: ne    (mm pr sqrt(km))                       (ne)

13. - 255. not used

Group no. 0 is used as a signal for undefined.

The conversion factor is used to give a unique 
internal representation:

group 1 and 2: radians
group 4:       metres
group 8:       geopotential units
group 10:      gal
group 12:      units of 1.0

The conversion factor is a multiplicator when the
conversion is to the unique interenal representation,
and a divisor when internal units are converted to
the external representation.

It is thus quite feasible to take input in one unit
and give output in any other sensible unit.

*/

#endif


/* typ_dec.h  version 1.1           # page 3   22 Feb 1994 13 13 */


#if !defined(H_GR_TP_FC)
#define      H_GR_TP_FC

#include    <math.h>

#ifndef    M_PI
#include   "kms_math.h"
#endif

struct na_gr_tp_fc {
  char        na[5];
  unsigned char     gr; /* group */
  unsigned char     tp; /* type  */
  short             dd; /* default digits bef. dec.-point */
  short             md; /* max no. digits bef. dec.-point */
  double            fc;  /* conv.  factor from int to ext */
};

static struct na_gr_tp_fc  ngtf[] = {
   {"sx",    1,  1, 7, 10, 180.0*60.0*60.0/M_PI},
   {"tm",    1,  2, 6,  9,  12.0*60.0*60.0/M_PI},
   {"N" ,    1,  3, 7, 10, 180.0*60.0*60.0/M_PI},
   {"S" ,    1,  4, 7, 10, 180.0*60.0*60.0/M_PI},
   {"E" ,    1,  5, 7, 10, 180.0*60.0*60.0/M_PI},
   {"W" ,    1,  6, 7, 10, 180.0*60.0*60.0/M_PI},
   {"nt",    1,  8, 5,  8, 180.0*60.0     /M_PI},
   {"mas",   1,  9, 2, 10, 180.0*60.0*60.0*1000.0/M_PI},

   {"dg",    2,  1, 3,  6, 180.0          /M_PI},
   {"asx",   2,  2, 2,  5, 180.0*60.0*60.0/M_PI},
   {"rad",   2,  3, 2,  5, 1.},
   {"yrad",  2,  4, 6,  9, 1.0e+6},
   {"cc",    2,  5, 7, 10, 2.0e+6/M_PI},
   {"gr",    2,  6, 3,  6, 2.0e+2/M_PI},
   {"gon",   2,  7, 3,  6, 2.0e+2/M_PI},
   {"mgon",  2,  8, 3,  6, 2.0e+5/M_PI},
   {"ts",    2,  9, 4,  7, 3200.0/M_PI},
   {"prad",  2, 10, 6,  9, 1.0e+9},

   {"m ",    4,  1, 7, 15, 1.},
   {"m",     4,  1, 7, 15, 1.},
   {"my",    4,  2, 3, 15, 1.0e+6},
   {"mm",    4,  3, 3, 15, 1.0e+3},
   {"cm",    4,  4, 3, 15, 1.0e+2},
   {"dm",    4,  5, 3, 15, 1.0e+1},
   {"km",    4,  6, 4, 15, 1.0e-3},
   {"nm",    4,  7, 4, 15, 1.0/1852.0},

   {"aum",   5,  1,14, 14, 149597870691.0},
   {"aus",   5,  1,14, 14, 499.0047838061},
   {"aly",   5,  1,14, 14, 9460730472580800.0},

   {"gpu",   8,  1, 4, 15, 1.},
   {"mpu",   8,  2, 4, 15, 1.0e+3},
   {"ypu",   8,  3, 4, 15, 1.0e+6},

   {"gal",  10,  1, 4, 15, 1.},
   {"mgal", 10,  2, 4, 15, 1.0e+3},
   {"ygal", 10,  3, 4, 15, 1.0e+6},
   {"kgal", 10,  4, 4, 15, 1.0e-3},

   {"ppm",  12,  1, 4, 15, 1.0e+6},
   {"ppb",  12,  2, 4, 15, 1.0e+9},
   {"ne",   12,  3, 4, 15, 31622.77660168379331995},
    /*                     1000 * sqrt(1000)   */

   {"",      0,  0, 0,  0, 0.0}  /* stop record */
};

#endif

